@@ -3,9 +3,8 @@
     Copyright (C) 1997,1998  Matt Kimball
 
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    it under the terms of the GNU General Public License version 2 as 
+    published by the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,6 +28,7 @@
 #include <math.h>
 #include <errno.h>
 
+#include "mtr.h"
 #include "display.h"
 #include "dns.h"
 #include "net.h"
@@ -43,7 +43,9 @@ static struct timeval intervaltime;
 int display_offset = 0;
 
 
-void select_loop() {
+#define GRACETIME (5 * 1000*1000)
+
+void select_loop(void) {
   fd_set readfd;
   int anyset = 0;
   int maxfd = 0;
@@ -51,8 +53,11 @@ void select_loop() {
   int NumPing = 0;
   int paused = 0;
   struct timeval lasttime, thistime, selecttime;
+  struct timeval startgrace, stopgrace;
   int dt;
   int rv; 
+  int graceperiod = 0;
+
 
   gettimeofday(&lasttime, NULL);
 
@@ -87,7 +92,6 @@ void select_loop() {
 	selecttime.tv_usec = 0;
       
 	rv = select(maxfd, (void *)&readfd, NULL, NULL, &selecttime);
-
       } else {
 	if(Interactive) display_redraw();
 
@@ -97,10 +101,21 @@ void select_loop() {
 	   (thistime.tv_sec == lasttime.tv_sec + intervaltime.tv_sec &&
 	    thistime.tv_usec >= lasttime.tv_usec + intervaltime.tv_usec)) {
 	  lasttime = thistime;
-	  if(NumPing >= MaxPing && (!Interactive || ForceMaxPing))
-	    return;
-	  if (net_send_batch())
-	    NumPing++;
+	  if (!graceperiod) {
+	    if(NumPing >= MaxPing && (!Interactive || ForceMaxPing)) {
+	      graceperiod=1;
+              startgrace=thistime;
+ 	      //gettimeofday (&startgrace, NULL);
+    	    }
+	    if (net_send_batch())
+	      NumPing++;
+          }
+	}
+	if (graceperiod) {
+	 // gettimeofday(&thistime, NULL);
+	  dt =        (thistime.tv_usec - startgrace.tv_usec) + 
+	    1000000 * (thistime.tv_sec - startgrace.tv_sec);
+	  if (dt > GRACETIME) return;
 	}
 
 	selecttime.tv_usec = (thistime.tv_usec - lasttime.tv_usec);
@@ -116,11 +131,13 @@ void select_loop() {
 	  selecttime.tv_usec += 1000000;
 	}
 
-	if ((selecttime.tv_sec > (time_t)dnsinterval) ||
-	    ((selecttime.tv_sec == (time_t)dnsinterval) &&
-	     (selecttime.tv_usec > ((time_t)(dnsinterval * 1000000) % 1000000)))) {
-	  selecttime.tv_sec = (time_t)dnsinterval;
-	  selecttime.tv_usec = (time_t)(dnsinterval * 1000000) % 1000000;
+	if (dns) {
+	  if ((selecttime.tv_sec > (time_t)dnsinterval) ||
+	      ((selecttime.tv_sec == (time_t)dnsinterval) &&
+	       (selecttime.tv_usec > ((time_t)(dnsinterval * 1000000) % 1000000)))) {
+	    selecttime.tv_sec = (time_t)dnsinterval;
+	    selecttime.tv_usec = (time_t)(dnsinterval * 1000000) % 1000000;
+	  }
 	}
 
 	rv = select(maxfd, (void *)&readfd, NULL, NULL, &selecttime);
